-------------------------------------------------------------------------------
--
-- Title       : aes_tb
-- Creator     : Øivind Ekelund
-- Date        : 25.03.09
--
-- Description : Testbench for power estimation
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
    
entity aes_tb is
end aes_tb;


architecture testbench of aes_tb is

constant PERIOD: time := 31 ns;
constant NR_ROUNDS: integer := 100;


signal clk, clk_en, reset_n, aes256, start, decrypt, stop, keybufen : std_logic := '0';
signal done, running : std_logic;

component aes_toplevel
   port( clk, clk_en, reset_n, aes256, start, decrypt, stop, keybufen : in 
         std_logic;  done, running : out std_logic);
end component;
        
 
begin

--AES toplevel module
u_aes_toplevel: aes_toplevel
   port map(
      clk      => clk,
      clk_en   => clk_en,
      reset_n  => reset_n,
      aes256   => aes256,
      start    => start,
      decrypt  => decrypt,
      stop     => stop,
      keybufen => keybufen,
      done     => done,
      running  => running
   );       
        

clock: process is
begin
    wait for PERIOD/2;
    clk <= '1';
    wait for PERIOD/2;
    clk <= '0';
end process clock;  

reset: process is
begin
    wait for PERIOD;
    reset_n <= '1';
    wait;
end process reset;

stimuli: process is
begin
    clk_en   <= '1';
    
    aes256   <= '0';
    decrypt  <= '0';
    keybufen <= '0';
    
    --Wait for reset 
    wait for 2*PERIOD;
    
    --Main loop, doing NR_ROUNDS encryptions
    for i in 0 to NR_ROUNDS-1 loop
       
       --Start
       --start <= '1';
       wait for PERIOD;
       start <= '0';
    
       --Wait for completion
       wait until done = '1';
       wait for PERIOD;
       
   end loop;
    
   wait;
end process stimuli;
  
end testbench;


CONFIGURATION aes_config OF aes_tb IS
   for testbench
	   for u_aes_toplevel : aes_toplevel
		   use entity work.aes_toplevel(syn_verilog);
      end for;
   end for;
END aes_config;
