// *****************************************************************************
//
// Title       : aes_testbench
// Creator     : Øivind Ekelund
// Date        : 23.05.09
//
// Description : Verification of AES128 en- and decryption
//               Initial data and key is 0
//               Encryption is performed NR_ROUND times
//               Decryption is performed NR_ROUND times
// *****************************************************************************


module aes_testbench();

parameter PERIOD = 10;
parameter NR_ROUNDS = 100;

reg clk, clk_en, reset_n, aes256, keybufen, decrypt, start, stop;
wire done, running;

reg [127:0] goldendataenc[0 : NR_ROUNDS];
reg [127:0] goldenkeyenc[0 : NR_ROUNDS];

integer i=0;

//Clock
always #(PERIOD/2) begin
    clk = 1;
    #(PERIOD/2)
    clk = 0;
end

//Reset and clock enable
initial begin
    $readmemh("goldendataenc.txt", goldendataenc);    
    $readmemh("goldenkeyenc.txt", goldenkeyenc);
    clk_en = 1;
    #PERIOD
    reset_n = 0;
    #PERIOD
    reset_n = 1;
end

//Teststimuli
initial begin  
    start       = 0;
    aes256      = 0;
    decrypt     = 0;
    keybufen    = 0;
    stop        = 0;
end

always @ (negedge done) begin
    if(!decrypt) begin
       if (i==0) begin //First round of simulation
          i = i+1; 
          #(3*PERIOD)
	       start = 1;
          #(2*PERIOD)
          start = 0;  
       end 
       else if(i<(NR_ROUNDS) && i>0) begin //Check results
          if(u_aes_toplevel.data_in != goldendataenc[i]) $display("***ERROR***, data is: %h, should be: %h", u_aes_toplevel.data_in, goldendataenc[i]);
          if(u_aes_toplevel.key_in != goldenkeyenc[i]) $display("***ERROR***, key is: %h, should be: %h", u_aes_toplevel.key_in, goldenkeyenc[i]);
          #(1*PERIOD)
	       start = 1;
          #(2*PERIOD)
          start = 0; 
          i = i+1; 	
       end
       else begin //Check last encryption result          
          if(u_aes_toplevel.data_in != goldendataenc[i]) $display("***ERROR***, data is: %h, should be: %h", u_aes_toplevel.data_in, goldendataenc[i]);
          if(u_aes_toplevel.key_in != goldenkeyenc[i]) $display("***ERROR***, key is: %h, should be: %h", u_aes_toplevel.key_in, goldenkeyenc[i]);
          $display("***VERIFICATION, AES128 ENCRYPTION COMPLETE***");
          
          //Start verification AES128 decryption
          decrypt = 1;          
          i = i-1;
          #(1*PERIOD)
	       start = 1;
          #(2*PERIOD)
          start = 0; 
       end
       
   end
   
   else if(decrypt) begin
              if (i==0) begin //Check last result
                 if(u_aes_toplevel.data_in != goldendataenc[i]) $display("***ERROR***, data is: %h, should be: %h", u_aes_toplevel.data_in, goldendataenc[i]);
                 if(u_aes_toplevel.key_in != goldenkeyenc[i]) $display("***ERROR***, key is: %h, should be: %h", u_aes_toplevel.key_in, goldenkeyenc[i]);
                 $display("***VERIFICATION, AES128 DECRYPTION COMPLETE***");

              end 
              else if(i<(NR_ROUNDS) && i>0) begin //Check results
                 if(u_aes_toplevel.data_in != goldendataenc[i]) $display("***ERROR***, data is: %h, should be: %h", u_aes_toplevel.data_in, goldendataenc[i]);
                 if(u_aes_toplevel.key_in != goldenkeyenc[i]) $display("***ERROR***, key is: %h, should be: %h", u_aes_toplevel.key_in, goldenkeyenc[i]);
                 #(1*PERIOD)
	              start = 1;
                 #(2*PERIOD)
                 start = 0; 
                 i = i-1; 	
              end   
   end
end

//DUT
aes_toplevel u_aes_toplevel(
    .clk        (clk),
    .reset_n    (reset_n),
    .clk_en     (clk_en),
    .start      (start),
    .stop       (stop),
    .decrypt    (decrypt),
    .keybufen   (keybufen),
    .aes256     (aes256),
    
    .done       (done),
    .running    (running)
);

endmodule