// *****************************************************************************
//
// Title       : aes_toplevel
// Creator     : Øivind Ekelund
// Date        : 25.03.09
//
// Description : AES module containing AES core and registers
//
// *****************************************************************************


module aes_toplevel(
   input wire   clk,
   input wire   clk_en,
   input wire   reset_n,
   
   input wire   aes256,           //Enablesignal for AES256
   input wire   start,            //Startsignal for AES Core
   input wire   decrypt,          //Enable decryption
   input wire   stop,             //Stop en/decryption in progress
   input wire   keybufen,         //Enable keybuffering

   output wire  done,             //Signals complete en/decryption
   output wire  running           //Asserted when AES core is working
);


///////////////////////////////////////////
//Signal declarations
///////////////////////////////////////////
wire [127:0]  data_in, data_out;
wire [127:0]  key_in, key_out;
wire [3:0]    key_we;
wire [3:0]    data_we;


///////////////////////////////////////////
//Data- and keyregisters
///////////////////////////////////////////
dataregister
   u_dataregister(
       .clk      (clk),
       .clk_en   (clk_en),
       .reset_n  (reset_n),
       .data_in  (data_out),
       .data_we  (data_we),
       .key_in   (key_out),
       .key_we   (key_we),
       .data_out (data_in),
       .key_out  (key_in)
   );

///////////////////////////////////////////
//AES core
///////////////////////////////////////////
v5_aes_core
   u_aes_core(
       .clk_core     (clk),
       .clken_core   (clk_en),
       .reset_core_n (reset_n),
       .data_in      (data_in),          //Data in from registers
       .key_in       (key_in),           //Key in from registers
       .aes256       (aes256),           //Enablesignal for AES256
       .start        (start),            //Startsignal for AES Core
       .decrypt      (decrypt),          //Enable decryption
       .stop         (stop),             //Stop en/decryption in progress
       .keybufen     (keybufen),         //Enable keybuffering
       .data_out     (data_out),         //Data to registers
       .key_out      (key_out),          //Key to registers
       .data_we      (data_we),          //Data write enable for dataregister
       .key_we       (key_we),           //Key write enable for keyregister
       .done         (done),             //Signals complete en/decryption
       .running      (running)           //Asserted when AES core is working
   );

endmodule
