
// *****************************************************************************
//
// Title       : dp128_aes_core
// Creator     : Øivind Ekelund 
// Date        : 25.03.09
//
// Description : AES core, containing sequencer and datapath
//
// *****************************************************************************

module dp128_aes_core
  (
   input wire            clk_core,
   input wire            clken_core,
   input wire            reset_core_n,
   input wire   [127:0]  data_in,          //Data in from registers
   input wire   [127:0]  key_in,           //Key in from registers
   input wire            aes256,           //Enablesignal for AES256
   input wire            start,            //Startsignal for AES Core
   input wire            decrypt,          //Enable decryption
   input wire            stop,             //Stop en/decryption in progress
   input wire            keybufen,         //Enable keybuffering
   output wire  [127:0]  data_out,         //Data to registers
   output wire  [127:0]  key_out,          //Key to registers
   output wire  [3:0]    data_we,          //Data write enable for dataregister
   output wire  [3:0]    key_we,           //Key write enable for keyregister
   output wire           done,             //Signals complete en/decryption
   output wire           running           //Asserted when AES core is working
   );

   // *****************************************************************************
   // Signal declarations
   // *****************************************************************************
   wire [7:0]            rcon;
   wire                  sb_mux;
   wire                  dpout_mux;
   wire                  addrk_mux;
   wire [1:0]            shrow_mux;

   // *****************************************************************************
   // AES Datapath
   // *****************************************************************************
   
   dp128_aes_datapath 
     u_datapath(/*AUTOINST*/
                // Outputs
                .data_out               (data_out[127:0]),
                .key_out                (key_out[127:0]),
                // Inputs
                .data_in                (data_in[127:0]),
                .key_in                 (key_in[127:0]),
                .addrk_mux              (addrk_mux),
                .shrow_mux              (shrow_mux[1:0]),
                .decrypt                (decrypt),
                .sb_mux                 (sb_mux),
                .dpout_mux              (dpout_mux),
                .rcon                   (rcon[7:0]));
   
   // *****************************************************************************
   // AES Sequencer
   // *****************************************************************************
   
   dp128_aes_sequencer 
     u_sequencer(/*AUTOINST*/
                 // Outputs
                 .running               (running),
                 .addrk_mux             (addrk_mux),
                 .sb_mux                (sb_mux),
                 .dpout_mux             (dpout_mux),
                 .shrow_mux             (shrow_mux[1:0]),
                 .rcon                  (rcon[7:0]),
                 .data_we               (data_we[3:0]),
                 .key_we                (key_we[3:0]),
                 .done                  (done),
                 // Inputs
                 .clk_core              (clk_core),
                 .clken_core            (clken_core),
                 .reset_core_n          (reset_core_n),
                 .start                 (start),
                 .decrypt               (decrypt),
                 //.aes256                (aes256),
                 .stop                  (stop)/*,
                 .keybufen              (keybufen)*/);
   
   
endmodule
