// *****************************************************************************
//
// Title       : d038_keypath
// Creator     : Øivind Ekelund
// Date        : 25.03.09
//
// Description : This module contains the logic needed for Key-Expansion
//
// *****************************************************************************

module d038_aes_keypath(
                    input wire  [127:0] key_in,
                    input wire  [31:0] from_sb,
                    input wire  decrypt,
                    output wire [127:0] key_out,
                    output wire [31:0] to_sb
                    );

   // *****************************************************************************
   // XOR-gates and muxes used in Key Expansion
   // *****************************************************************************
   assign to_sb            = (decrypt) ? key_out[31:0] : key_in[31:0];
   
   assign key_out[31:0]    = (decrypt) ? key_in[31:0] ^ key_in[63:32]
                              : key_in[31:0] ^ key_out[63:32];
                               
   assign key_out[63:32]   = (decrypt) ? key_in[63:32] ^ key_in[95:64]
                              : key_in[63:32] ^ key_out[95:64];
                              
   assign key_out[95:64]   = (decrypt) ? key_in[95:64] ^ key_in[127:96]
                              : key_in[95:64] ^ key_out[127:96];
                              
   assign key_out[127:96]  = key_in[127:96] ^ from_sb;
   
endmodule
