// *****************************************************************************
//
// Title       : d038_aes_core
// Creator     : Øivind Ekelund
// Date        : 25.03.09
//
// Description : AES core, containing sequencer and datapath
//
// *****************************************************************************

module d038_aes_core(
                     input wire   clk_core,
                     input wire   clken_core,
                     input wire   reset_core_n,
                     input wire   [127:0] data_in,  //Data in from registers
                     input wire   [127:0] key_in,   //Key in from registers
                     input wire   aes256,           //Enablesignal for AES256
                     input wire   start,            //Startsignal for AES Core
                     input wire   decrypt,          //Enable decryption
                     input wire   stop,             //Stop en/decryption in progress
                     input wire   keybufen,         //Enable keybuffering
                     output wire  [127:0] data_out, //Data to registers
                     output wire  [127:0] key_out,  //Key to registers
                     output wire  [3:0] data_we,    //Data write enable for dataregister
                     output wire  [3:0] key_we,     //Key write enable for keyregister
                     output wire  done,             //Signals complete en/decryption
                     output wire  running           //Asserted when AES core is working
                     );

   // *****************************************************************************
   // Signal declarations
   // *****************************************************************************
   wire [7:0]                     rcon;
   wire [3:0]                     data_out_mux;
   wire [1:0]                     key_mux;
   wire [1:0]                     data_mux;
   wire                           sb_mux;
   wire [1:0]                     dpout_mux;
   wire                           sb_inv;
   wire                           mc_inv;
   wire                           addrk_mux;
   wire                           keymc_mux;


   // *****************************************************************************
   // AES Datapath
   // *****************************************************************************
   
   d038_datapath 
     u_datapath(/*AUTOINST*/
                // Outputs
                .data_out               (data_out[127:0]),
                .key_out                (key_out[127:0]),
                // Inputs
                .data_in                (data_in[127:0]),
                .key_in                 (key_in[127:0]),
                .sb_inv                 (sb_inv),
                .mc_inv                 (mc_inv),
                .addrk_mux              (addrk_mux),
                .decrypt                (decrypt),
                .data_mux               (data_mux[1:0]),
                .key_mux                (key_mux[1:0]),
                .keymc_mux              (keymc_mux),
                .sb_mux                 (sb_mux),
                .dpout_mux              (dpout_mux[1:0]),
                .data_out_mux           (data_out_mux[3:0]),
                .rcon                   (rcon[7:0]));
   

   // *****************************************************************************
   // AES Sequencer
   // *****************************************************************************
   
   d038_aes_sequencer 
     u_sequencer(/*AUTOINST*/
                 // Outputs
                 .sb_inv                (sb_inv),
                 .mc_inv                (mc_inv),
                 .running               (running),
                 .addrk_mux             (addrk_mux),
                 .data_mux              (data_mux[1:0]),
                 .key_mux               (key_mux[1:0]),
                 .keymc_mux             (keymc_mux),
                 .sb_mux                (sb_mux),
                 .dpout_mux             (dpout_mux[1:0]),
                 .data_out_mux          (data_out_mux[3:0]),
                 .rcon                  (rcon[7:0]),
                 .data_we               (data_we[3:0]),
                 .key_we                (key_we[3:0]),
                 .done                  (done),
                 // Inputs
                 .clk_core              (clk_core),
                 .clken_core            (clken_core),
                 .reset_core_n          (reset_core_n),
                 .start                 (start),
                 .decrypt               (decrypt),
                 //.aes256                (aes256),
                 .stop                  (stop)/*,
                 .keybufen              (keybufen)*/);
   
   
endmodule
