// *****************************************************************************
//
// Title       : v3_aes_mixcolumns
// Creator     : ivind Ekelund
// Date        : 25.03.09
//
// Description : Module performing the (Inv)MixColumns part in AES 
//
// *****************************************************************************

module v3_mixcolumns
  (
   input wire  [31:0]    mc_in,
   input wire  [31:0]    roundkey,
   input wire            inverse, 
   output wire [31:0]    mc_out
   );

   // *****************************************************************************
   // Signal declarations
   // *****************************************************************************
   
   wire [31:0]           mux_out;
   wire [31:0]           minv2_out;
   wire [7:0]            minv2_tmp1;
   wire [7:0]            minv2_tmp2;
   wire [7:0]            m_tmp1;
   wire [7:0]            m_tmp2;
   wire [7:0]            m_tmp3;
   wire [7:0]            m_tmp4;
   wire [31:0]           mcin_xor_rkey;

   // *****************************************************************************
   // Logics for computation of MixColumns and InvMixColumns
   // *****************************************************************************
   
   //assign mcin_xor_rkey = mc_in ^ roundkey;
   xor32 u_xor32(
      .xor_ina(mc_in),
      .xor_inb(roundkey),
      .xor_out(mcin_xor_rkey)
   );
             
   assign minv2_tmp1 = xtime(xtime(mcin_xor_rkey[31:24] ^ mcin_xor_rkey[15:8]));
   assign minv2_tmp2 = xtime(xtime(mcin_xor_rkey[23:16] ^ mcin_xor_rkey[7:0]));
   
   assign minv2_out[31:24] = minv2_tmp1 ^ mcin_xor_rkey[31:24];
   assign minv2_out[23:16] = minv2_tmp2 ^ mcin_xor_rkey[23:16];
   assign minv2_out[15:8]  = minv2_tmp1 ^ mcin_xor_rkey[15:8];
   assign minv2_out[7:0]   = minv2_tmp2 ^ mcin_xor_rkey[7:0];   

   //Mux, choosing en- or decryption
   assign mux_out = (inverse) ? minv2_out : mc_in;
   
   assign m_tmp1 = xtime(mux_out[23:16]) ^ mux_out[15:8]  ^ mux_out[7:0];
   assign m_tmp2 = xtime(mux_out[7:0])   ^ mux_out[31:24] ^ mux_out[23:16];
   assign m_tmp3 = xtime(mux_out[31:24]);
   assign m_tmp4 = xtime(mux_out[15:8]);
   
   assign mc_out[31:24] = m_tmp3 ^ m_tmp1 ^ mux_out[23:16];
   assign mc_out[23:16] = m_tmp4 ^ m_tmp1 ^ mux_out[31:24];
   assign mc_out[15:8]  = m_tmp4 ^ m_tmp2 ^ mux_out[7:0];
   assign mc_out[7:0]   = m_tmp3 ^ m_tmp2 ^ mux_out[15:8];
   

   //Function performing multiplication by 2 in GF(256)
   function automatic [7:0] xtime;
      input [7:0]        x;
      begin
         if (x[7]) 
           xtime = {x[6:0], 1'b0} ^ 8'h1B;
         else 
           xtime = {x[6:0], 1'b0};
      end
   endfunction
   
endmodule

module xor32(
   input wire [31:0] xor_ina,
   input wire [31:0] xor_inb,
   output wire [31:0] xor_out
);

assign xor_out = xor_ina ^ xor_inb;

endmodule
