// *****************************************************************************
//
// Title       : dataregister
// Creator     : Øivind Ekelund
// Date        : 25.03.09
//
// Description : Data- and keyregister
//
// *****************************************************************************

module dataregister(
    input wire clk,
    input wire clk_en,
    input wire reset_n,
    input wire [127:0] data_in,
    input wire [3:0] data_we,
    input wire [255:0] key_in,
    input wire [7:0] key_we,
    output wire [127:0] data_out,
    output wire [255:0] key_out
    );
    
    //Internal registers
    reg [127:0] data_reg;
    reg [255:0] key_reg;
    
    assign data_out = data_reg;
    assign key_out = key_reg;
    
    always @ (posedge clk, negedge reset_n) begin
            if (!reset_n) begin
                    data_reg <= 0;
                    key_reg <= 0;
                                    
                end
            else if (clk_en) begin
                    //Update dataregister
                    if (data_we[0]) data_reg[31:0] <= data_in[31:0];
                    if (data_we[1]) data_reg[63:32] <= data_in[63:32];
                    if (data_we[2]) data_reg[95:64] <= data_in[95:64];
                    if (data_we[3]) data_reg[127:96] <= data_in[127:96];
                    //Update keyregister
                    if (key_we[0]) key_reg[31:0] <= key_in[31:0];
                    if (key_we[1]) key_reg[63:32] <= key_in[63:32];
                    if (key_we[2]) key_reg[95:64] <= key_in[95:64];
                    if (key_we[3]) key_reg[127:96] <= key_in[127:96];
                    if (key_we[4]) key_reg[159:128] <= key_in[159:128];
                    if (key_we[5]) key_reg[191:160] <= key_in[191:160];
                    if (key_we[6]) key_reg[223:192] <= key_in[223:192];
                    if (key_we[7]) key_reg[255:224] <= key_in[255:224];
                end
        end
    
endmodule
